@@ -1,9 +1,12 @@
-//! [Church-encoded numerals](https://en.wikipedia.org/wiki/Church_encoding#Church_numerals)
+//! [Church-encoded numerals](https://en.wikipedia.org/wiki/Church_encoding#Church_numerals) and
+//! arithmetic operations
 
 use term::*;
 use term::Term::*;
 use term::Error::*;
 use booleans::*;
+//use pair::pair;
+//use combinators::y;
 
 /// Produces a Church-encoded zero.
 ///
@@ -30,7 +33,13 @@ pub fn zero() -> Term { abs(abs(Var(1))) }
 ///
 /// assert_eq!(normalize(is_zero().app(zero())), tru());
 /// ```
-pub fn is_zero() -> Term { abs(Var(1).app(abs(fls())).app(tru())) }
+pub fn is_zero() -> Term {
+    abs(
+        Var(1)
+        .app(abs(fls()))
+        .app(tru())
+    )
+}
 
 /// Produces a Church-encoded one.
 ///
@@ -42,7 +51,11 @@ pub fn is_zero() -> Term { abs(Var(1).app(abs(fls())).app(tru())) }
 ///
 /// assert_eq!(format!("{}", one()), "λλ21");
 /// ```
-pub fn one() -> Term { abs(abs(Var(2).app(Var(1)))) }
+pub fn one() -> Term {
+    abs(abs(
+        Var(2).app(Var(1))
+    ))
+}
 
 /// Applied to a Church-encoded number it produces its successor.
 ///
@@ -55,7 +68,15 @@ pub fn one() -> Term { abs(abs(Var(2).app(Var(1)))) }
 ///
 /// assert_eq!(normalize(succ().app(zero())), one());
 /// ```
-pub fn succ() -> Term { abs(abs(abs(Var(2).app(Var(3).app(Var(2)).app(Var(1)))))) }
+pub fn succ() -> Term {
+    abs(abs(abs(
+        Var(2).app(
+            Var(3)
+            .app(Var(2))
+            .app(Var(1))
+        )
+    )))
+}
 
 /// Applied to two Church-encoded numbers it produces their sum.
 ///
@@ -68,7 +89,13 @@ pub fn succ() -> Term { abs(abs(abs(Var(2).app(Var(3).app(Var(2)).app(Var(1)))))
 ///
 /// assert_eq!(normalize(plus().app(zero()).app(one())), one());
 /// ```
-pub fn plus() -> Term { abs(abs(abs(abs(Var(4).app(Var(2)).app(Var(3).app(Var(2)).app(Var(1))))))) }
+pub fn plus() -> Term {
+    abs(abs(abs(abs(
+        Var(4)
+        .app(Var(2))
+        .app(Var(3).app(Var(2)).app(Var(1)))
+    ))))
+}
 
 /// Applied to two Church-encoded numbers it produces their product.
 ///
@@ -81,7 +108,11 @@ pub fn plus() -> Term { abs(abs(abs(abs(Var(4).app(Var(2)).app(Var(3).app(Var(2)
 ///
 /// assert_eq!(normalize(mult().app(one()).app(one())), one());
 /// ```
-pub fn mult() -> Term { abs(abs(abs(Var(3).app(Var(2).app(Var(1)))))) }
+pub fn mult() -> Term {
+    abs(abs(abs(
+        Var(3).app(Var(2).app(Var(1)))
+    )))
+}
 
 /// Applied to two Church-encoded numbers it raises the first one to the power of the second one.
 ///
@@ -94,7 +125,11 @@ pub fn mult() -> Term { abs(abs(abs(Var(3).app(Var(2).app(Var(1)))))) }
 ///
 /// assert_eq!(normalize(pow().app(one()).app(one())), one());
 /// ```
-pub fn pow() -> Term { abs(abs(Var(1).app(Var(2)))) }
+pub fn pow() -> Term {
+    abs(abs(
+        Var(1).app(Var(2))
+    ))
+}
 
 /// Applied to a Church-encoded number it produces its predecessor.
 ///
@@ -127,14 +162,18 @@ pub fn pred() -> Term {
 ///
 /// assert_eq!(normalize(sub().app(one()).app(zero())), one());
 /// ```
-pub fn sub() -> Term { abs(abs(Var(1).app(pred()).app(Var(2)))) }
+pub fn sub() -> Term {
+    abs(abs(
+        Var(1).app(pred()).app(Var(2))
+    ))
+}
 
-/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether its
-/// first argument is less than the second one.
+/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether
+/// its first argument is less than the second one.
 ///
 /// LT := λab.NOT (LEQ b a) = λ λ NOT (LEQ 1 2)
 ///
-/// # Example
+/// # Examples
 /// ```
 /// use lambda_calculus::arithmetic::{zero, one, lt};
 /// use lambda_calculus::booleans::{tru, fls};
@@ -145,14 +184,18 @@ pub fn sub() -> Term { abs(abs(Var(1).app(pred()).app(Var(2)))) }
 /// assert_eq!(normalize(lt().app(zero()).app(one())),  tru());
 /// assert_eq!(normalize(lt().app(one()).app(zero())),  fls());
 /// ```
-pub fn lt() -> Term { abs(abs(not().app(leq().app(Var(1)).app(Var(2))))) }
+pub fn lt() -> Term {
+    abs(abs(
+        not().app(leq().app(Var(1)).app(Var(2)))
+    ))
+}
 
-/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether its
-/// first argument is less than or egual to the second one.
+/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether
+/// its first argument is less than or egual to the second one.
 ///
 /// LEQ := λmn.IS_ZERO (SUB m n) = λ λ IS_ZERO (SUB 2 1)
 ///
-/// # Example
+/// # Examples
 /// ```
 /// use lambda_calculus::arithmetic::{zero, one, leq};
 /// use lambda_calculus::booleans::{tru, fls};
@@ -163,14 +206,18 @@ pub fn lt() -> Term { abs(abs(not().app(leq().app(Var(1)).app(Var(2))))) }
 /// assert_eq!(normalize(leq().app(zero()).app(one())),  tru());
 /// assert_eq!(normalize(leq().app(one()).app(zero())),  fls());
 /// ```
-pub fn leq() -> Term { abs(abs(is_zero().app(sub().app(Var(2)).app(Var(1))))) }
+pub fn leq() -> Term {
+    abs(abs(
+        is_zero().app(sub().app(Var(2)).app(Var(1)))
+    ))
+}
 
-/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether its
-/// first argument is egual to the second one.
+/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether
+/// its first argument is egual to the second one.
 ///
 /// EQ := λmn.AND (LEQ m n) (LEQ n m) = λ λ AND (LEQ 2 1) (LEQ 1 2)
 ///
-/// # Example
+/// # Examples
 /// ```
 /// use lambda_calculus::arithmetic::{zero, one, eq};
 /// use lambda_calculus::booleans::{tru, fls};
@@ -189,12 +236,12 @@ pub fn eq() -> Term {
     ))
 }
 
-/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether its
-/// first argument is not egual to the second one.
+/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether
+/// its first argument is not egual to the second one.
 ///
 /// NEQ := λab.OR (NOT (LEQ a b)) (NOT (LEQ b a)) = λ λ OR (NOT (LEQ 2 1)) (NOT (LEQ 1 2))
 ///
-/// # Example
+/// # Examples
 /// ```
 /// use lambda_calculus::arithmetic::{zero, one, neq};
 /// use lambda_calculus::booleans::{tru, fls};
@@ -213,12 +260,12 @@ pub fn neq() -> Term {
     ))
 }
 
-/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether its
-/// first argument is greater than or egual to the second one.
+/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether
+/// its first argument is greater than or egual to the second one.
 ///
 /// GEQ := λab.LEQ b a = λ λ LEQ 1 2
 ///
-/// # Example
+/// # Examples
 /// ```
 /// use lambda_calculus::arithmetic::{zero, one, geq};
 /// use lambda_calculus::booleans::{tru, fls};
@@ -229,14 +276,18 @@ pub fn neq() -> Term {
 /// assert_eq!(normalize(geq().app(zero()).app(one())),  fls());
 /// assert_eq!(normalize(geq().app(one()).app(zero())),  tru());
 /// ```
-pub fn geq() -> Term { abs(abs(leq().app(Var(1)).app(Var(2)))) }
+pub fn geq() -> Term {
+    abs(abs(
+        leq().app(Var(1)).app(Var(2))
+    ))
+}
 
-/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether its
-/// first argument is greater than the second one.
+/// Applied to two Church-encoded numbers it returns a Church-encoded boolean indicating whether
+/// its first argument is greater than the second one.
 ///
 /// GT := λab.NOT (LEQ a b) = λ λ NOT (LEQ 2 1)
 ///
-/// # Example
+/// # Examples
 /// ```
 /// use lambda_calculus::arithmetic::{zero, one, gt};
 /// use lambda_calculus::booleans::{tru, fls};
@@ -247,8 +298,83 @@ pub fn geq() -> Term { abs(abs(leq().app(Var(1)).app(Var(2)))) }
 /// assert_eq!(normalize(gt().app(zero()).app(one())),  fls());
 /// assert_eq!(normalize(gt().app(one()).app(zero())),  tru());
 /// ```
-pub fn gt() -> Term { abs(abs(not().app(leq().app(Var(2)).app(Var(1))))) }
+pub fn gt() -> Term {
+    abs(abs(
+        not().app(leq().app(Var(2)).app(Var(1)))
+    ))
+}
+/*
+// FIXME: blows up RAM
+/// Applied to two Church-encoded numbers it returns a Church-encoded pair with the result of their
+/// division - the quotient and the remainder.
+///
+/// DIV := Y (λgqab.LT a b (PAIR q a) (g (SUCC q) (SUB a b) b)) ZERO =
+/// Y (λ λ λ λ LT 2 1 (PAIR 3 2) (4 (SUCC 3) (SUB 2 1) 1)) ZERO
+///
+/// # Example
+/// ```
+/// use lambda_calculus::arithmetic::{div, one, to_cnum};
+/// use lambda_calculus::pair::pair;
+/// use lambda_calculus::reduction::normalize;
+///
+/// assert_eq!(normalize(div().app(to_cnum(5)).app(to_cnum(2))),
+///            normalize(pair().app(to_cnum(2)).app(one())));
+/// ```
+pub fn div() -> Term {
+    y()
+    .app(
+        abs(abs(abs(abs(
+            lt()
+            .app(Var(2))
+            .app(Var(1))
+            .app(pair().app(Var(3)).app(Var(2)))
+            .app(
+                Var(4)
+                .app(succ().app(Var(3)))
+                .app(sub().app(Var(2)).app(Var(1)))
+                .app(Var(1))
+            )
+        ))))
+    )
+    .app(zero())
+}
+*/
 
+/*
+// FIXME: blows up RAM
+/// Applied to two Church-encoded numbers it returns a Church-encoded quotient of their division.
+///
+/// QUOT := y (λrab.LT a b ZERO (SUCC (r (SUB a b) b)))
+///
+/// # Example
+/// ```
+/// use lambda_calculus::arithmetic::{quot, to_cnum};
+/// use lambda_calculus::reduction::normalize;
+///
+/// assert_eq!(normalize(quot().app(to_cnum(6)).app(to_cnum(2))), to_cnum(3));
+/// ```
+pub fn quot() -> Term {
+    y().app(
+        abs(abs(abs(
+            lt()
+            .app(Var(2))
+            .app(Var(1))
+            .app(zero())
+            .app(
+                succ().app(
+                    Var(3)
+                    .app(
+                        sub()
+                        .app(Var(2))
+                        .app(Var(1))
+                    )
+                    .app(Var(1))
+                )
+            )
+        )))
+    )
+}
+*/
 impl Term {
     /// Returns the value of a Church-encoded number.
     ///
@@ -286,9 +412,9 @@ impl Term {
     /// ```
     /// use lambda_calculus::arithmetic::one;
     ///
-    /// assert!(one().is_number());
+    /// assert!(one().is_cnum());
     /// ```
-    pub fn is_number(&self) -> bool { self.value().is_ok() }
+    pub fn is_cnum(&self) -> bool { self.value().is_ok() }
 }
 
 /// Produces a Church-encoded term with a value of the given natural number.
@@ -326,12 +452,13 @@ mod test {
     fn church_successor() {
         assert_eq!(normalize(succ().app(zero())), one());
         assert_eq!(normalize(succ().app(one())), abs(abs(Var(2).app(Var(2).app(Var(1))))));
-        assert_eq!(normalize(succ().app(succ().app(succ().app(zero())))), abs(abs(Var(2).app(Var(2).app(Var(2).app(Var(1)))))));
+        assert_eq!(normalize(succ().app(succ().app(succ().app(zero())))),
+                   abs(abs(Var(2).app(Var(2).app(Var(2).app(Var(1)))))));
     }
 
     #[test]
     fn church_number_identification() {
-        for n in 0..5 { assert!(to_cnum(n).is_number()) }
+        for n in 0..5 { assert!(to_cnum(n).is_cnum()) }
     }
 
     #[test]
@@ -376,7 +503,7 @@ mod test {
         assert_eq!(normalize(pow().app(to_cnum(1)).app(to_cnum(6))), to_cnum(1));
         assert_eq!(normalize(pow().app(to_cnum(3)).app(to_cnum(2))), to_cnum(9));
         assert_eq!(normalize(pow().app(to_cnum(4)).app(to_cnum(1))), to_cnum(4));
-//      assert_eq!(normalize(pow().app(to_cnum(5)).app(zero())),     to_cnum(1)); // n^0 fails - why?
+//      assert_eq!(normalize(pow().app(to_cnum(5)).app(zero())),     to_cnum(1)); // n^0 fails
     }
 
     #[test]
