@@ -5,6 +5,9 @@ use term::Term::*;
 use term::Error::*;
 use booleans::*;
 use pair::*;
+use arithmetic::{zero, succ};
+use combinators::y;
+use std::ops::Index;
 
 /// Equivalent to fls(); produces a Church-encoded nil, the last link of a Church-encoded list.
 ///
@@ -21,17 +24,23 @@ pub fn nil() -> Term { fls() }
 
 /// Applied to a Church-encoded list it determines if it is empty.
 ///
-/// IS_NIL := λl.l (λhtd.FALSE) TRUE = λ 1 (λ λ λ FALSE) TRUE
+/// NULL := λl.l (λhtd.FALSE) TRUE = λ 1 (λ λ λ FALSE) TRUE
 ///
 /// # Example
 /// ```
-/// use lambda_calculus::list::{nil, is_nil};
+/// use lambda_calculus::list::{nil, null};
 /// use lambda_calculus::booleans::tru;
 /// use lambda_calculus::reduction::normalize;
 ///
-/// assert_eq!(normalize(is_nil().app(nil())), tru());
+/// assert_eq!(normalize(null().app(nil())), tru());
 /// ```
-pub fn is_nil() -> Term { abs(Var(1).app(abs(abs(abs(fls())))).app(tru())) }
+pub fn null() -> Term {
+    abs(
+        Var(1)
+        .app(abs(abs(abs(fls()))))
+        .app(tru())
+    )
+}
 
 /// Equivalent to pair(); applied to two terms it returns them encoded as a list.
 ///
@@ -44,7 +53,19 @@ pub fn is_nil() -> Term { abs(Var(1).app(abs(abs(abs(fls())))).app(tru())) }
 /// use lambda_calculus::list::{nil, cons};
 /// use lambda_calculus::reduction::normalize;
 ///
-/// let list_110_consed = normalize(cons().app(one()).app(cons().app(one()).app(cons().app(zero()).app(nil()))));
+/// let list_110_consed = normalize(
+///    cons()
+///    .app(one())
+///    .app(
+///        cons()
+///        .app(one())
+///        .app(
+///            cons()
+///            .app(zero())
+///            .app(nil())
+///           )
+///      )
+/// );
 /// let list_110_from_vec = Term::from(vec![one(), one(), zero()]);
 ///
 /// assert_eq!(list_110_consed, list_110_from_vec);
@@ -68,7 +89,8 @@ pub fn cons() -> Term { pair() }
 /// ```
 pub fn head() -> Term { first() }
 
-/// Equivalent to second(); applied to a Church-encoded list it returns a new list with all its elements but the first one.
+/// Equivalent to second(); applied to a Church-encoded list it returns a new list with all its
+/// elements but the first one.
 ///
 /// TAIL := SECOND
 ///
@@ -85,6 +107,109 @@ pub fn head() -> Term { first() }
 /// ```
 pub fn tail() -> Term { second() }
 
+/// Applied to a Church-encoded list it returns its Church-encoded length.
+///
+/// LENGTH := Y (λgcx.NULL x c (g (SUCC c) (SECOND x))) ZERO
+/// = Y (λλλ NULL 1 2 (3 (SUCC 2) (SECOND 1))) ZERO
+///
+/// # Example
+/// ```
+/// use lambda_calculus::term::Term;
+/// use lambda_calculus::list::{length, nil};
+/// use lambda_calculus::arithmetic::{zero, one, to_cnum};
+/// use lambda_calculus::reduction::normalize;
+///
+/// let list_4 = Term::from(vec![one(), one(), zero(), one()]);
+///
+/// assert_eq!(normalize(length().app(nil())),  zero());
+/// assert_eq!(normalize(length().app(list_4)), to_cnum(4));
+/// ```
+pub fn length() -> Term {
+    y()
+    .app(
+        abs(abs(abs(
+            null()
+            .app(Var(1))
+            .app(Var(2))
+            .app(
+                Var(3)
+                .app(succ().app(Var(2)))
+                .app(second().app(Var(1)))
+            )
+        )))
+    )
+    .app(zero())
+}
+
+/// Reverses a Church-encoded list.
+///
+/// REVERSE := Y (λgal.NULL l a (g (PAIR (FIRST l) a) (SECOND l))) NIL =
+/// Y (λ λ λ NULL 1 2 (3 (PAIR (FIRST 1) 2) (SECOND 1))) NIL
+///
+/// # Example
+/// ```
+/// use lambda_calculus::term::Term;
+/// use lambda_calculus::list::reverse;
+/// use lambda_calculus::arithmetic::{zero, one};
+/// use lambda_calculus::reduction::normalize;
+///
+/// let list = Term::from(vec![one(), one(), zero()]);
+///
+/// assert_eq!(normalize(reverse().app(list)), Term::from(vec![zero(), one(), one()]));
+/// ```
+pub fn reverse() -> Term {
+    y()
+    .app(
+        abs(abs(abs(
+            null()
+            .app(Var(1))
+            .app(Var(2))
+            .app(
+                Var(3)
+                .app(
+                    pair()
+                    .app(first().app(Var(1)))
+                    .app(Var(2))
+                )
+                .app(second().app(Var(1)))
+            )
+        )))
+    )
+    .app(nil())
+}
+
+/// Applied to a Church-encoded number `n` and `n` `Term`s it creates a Church-encoded list of
+/// those terms.
+///
+/// LIST := λn.n (λfax.f (PAIR x a)) REVERSE NIL = λ 1 (λ λ λ 3 (PAIR 1 2)) REVERSE NIL
+///
+/// # Example
+/// ```
+/// use lambda_calculus::term::Term;
+/// use lambda_calculus::list::list;
+/// use lambda_calculus::arithmetic::{zero, one, to_cnum};
+/// use lambda_calculus::reduction::normalize;
+///
+/// assert_eq!(normalize(list().app(to_cnum(3)).app(zero()).app(one()).app(one())),
+///            Term::from(vec![zero(), one(), one()]));
+/// ```
+pub fn list() -> Term {
+    abs(
+        Var(1)
+        .app(
+            abs(abs(abs(
+                Var(3).app(
+                    pair()
+                    .app(Var(1))
+                    .app(Var(2))
+                )
+            )))
+        )
+        .app(reverse())
+        .app(nil())
+    )
+}
+
 impl Term {
     /// Checks whether self is a Church-encoded empty list, i.e. nil().
     ///
@@ -100,6 +225,8 @@ impl Term {
 
     // Returns a reference to the last term of a Church-encoded list.
     fn last_ref(&self) -> Result<&Term, Error> {
+        if !self.is_pair() { return Err(NotAList) }
+
         let mut last_candidate = try!(self.snd_ref());
 
         while let Ok(ref second) = last_candidate.snd_ref() {
@@ -124,7 +251,8 @@ impl Term {
         self.last_ref() == Ok(&nil())
     }
 
-    /// Splits a Church-encoded list into a pair containing its first term and a list of all the other terms, consuming self.
+    /// Splits a Church-encoded list into a pair containing its first term and a list of all the
+    /// other terms, consuming self.
     ///
     /// # Example
     /// ```
@@ -143,7 +271,8 @@ impl Term {
         }
     }
 
-    /// Splits a Church-encoded list into a pair containing references to its first term and a to list of all the other terms.
+    /// Splits a Church-encoded list into a pair containing references to its first term and a to
+    /// list of all the other terms.
     ///
     /// # Example
     /// ```
@@ -162,7 +291,8 @@ impl Term {
         }
     }
 
-    /// Splits a Church-encoded list into a pair containing mutable references to its first term and a to list of all the other terms.
+    /// Splits a Church-encoded list into a pair containing mutable references to its first term
+    /// and a to list of all the other terms.
     ///
     /// # Example
     /// ```
@@ -171,7 +301,8 @@ impl Term {
     ///
     /// let mut list_110 = Term::from(vec![one(), one(), zero()]);
     ///
-    /// assert_eq!(list_110.uncons_ref_mut(), Ok((&mut one(), &mut Term::from(vec![one(), zero()]))));
+    /// assert_eq!(list_110.uncons_ref_mut(),
+    ///            Ok((&mut one(), &mut Term::from(vec![one(), zero()]))));
     /// ```
     pub fn uncons_ref_mut(&mut self) -> Result<(&mut Term, &mut Term), Error> {
         if !self.is_list() {
@@ -256,7 +387,8 @@ impl Term {
         Ok(try!(self.uncons_ref()).1)
     }
 
-    /// Returns a mutable reference to a list of all the terms of a Church-encoded list but the first one.
+    /// Returns a mutable reference to a list of all the terms of a Church-encoded list but the
+    /// first one.
     ///
     /// # Example
     /// ```
@@ -294,7 +426,8 @@ impl Term {
         Ok(n)
     }
 
-    /// Adds a term to the beginning of a Church-encoded list and returns the new list. Consumes self and the argument.
+    /// Adds a term to the beginning of a Church-encoded list and returns the new list. Consumes
+    /// self and the argument.
     ///
     /// # Example
     /// ```
@@ -322,7 +455,7 @@ impl Term {
     /// assert_eq!(list_110, Term::from(vec![one(), zero()]));
     /// ```
     pub fn pop(&mut self) -> Result<Term, Error> {
-        let (head, tail) = try!(self.clone().uncons());
+        let (head, tail) = try!(self.clone().uncons()); // TODO: drop clone()
         *self = tail;
 
         Ok(head)
@@ -353,6 +486,24 @@ impl Iterator for Term {
     }
 }
 
+impl Index<usize> for Term {
+    type Output = Term;
+
+    fn index(&self, i: usize) -> &Self::Output {
+        if !self.is_list() { panic!("attempting to index something that is not a list!") }
+
+        if i == 0 { return self.head_ref().unwrap() } // safe - checked above
+
+        let mut candidate = self.snd_ref().expect("index out of bounds!");
+
+        for _ in 1..i {
+            candidate = candidate.snd_ref().expect("index out of bounds!")
+        }
+
+        candidate.head_ref().unwrap()
+    }
+}
+
 #[cfg(test)]
 mod test {
     use super::*;
@@ -370,7 +521,9 @@ mod test {
     #[test]
     fn list_push() {
         let list_pushed = nil().push(zero()).push(one()).push(one());
-        let list_consed = normalize(cons().app(one()).app(cons().app(one()).app(cons().app(zero()).app(nil()))));
+        let list_consed = normalize(
+            cons().app(one()).app(cons().app(one()).app(cons().app(zero()).app(nil())))
+        );
 
         assert_eq!(list_pushed, list_consed);
     }
@@ -397,17 +550,19 @@ mod test {
 
     #[test]
     fn list_operations() {
-        let list_three_five_four = Term::from(vec![to_cnum(3), to_cnum(5), to_cnum(4)]);
+        let list_354 = Term::from(vec![to_cnum(3), to_cnum(5), to_cnum(4)]);
 
-        assert!(list_three_five_four.is_list());
+        assert!(list_354.is_list());
 
-        assert_eq!(list_three_five_four.head_ref(), Ok(&to_cnum(3)));
-        assert_eq!(list_three_five_four.tail_ref(), Ok(&Term::from(vec![to_cnum(5), to_cnum(4)])));
+        assert_eq!(list_354.head_ref(), Ok(&to_cnum(3)));
+        assert_eq!(list_354.tail_ref(), Ok(&Term::from(vec![to_cnum(5), to_cnum(4)])));
 
-        assert_eq!(list_three_five_four.tail_ref().and_then(|t| t.head_ref()), Ok(&to_cnum(5)));
-        assert_eq!(list_three_five_four.tail_ref().and_then(|t| t.tail_ref()).and_then(|t| t.head_ref()), Ok(&to_cnum(4)));
+        assert_eq!(list_354.tail_ref().and_then(|t| t.head_ref()), Ok(&to_cnum(5)));
+        assert_eq!(list_354.tail_ref()
+                   .and_then(|t| t.tail_ref())
+                   .and_then(|t| t.head_ref()), Ok(&to_cnum(4)));
 
-        let unconsed = list_three_five_four.uncons();
+        let unconsed = list_354.uncons();
         assert_eq!(unconsed, Ok((to_cnum(3), Term::from(vec![to_cnum(5), to_cnum(4)]))));
     }
 
@@ -432,4 +587,15 @@ mod test {
         assert_eq!(iter.next(), Some(zero()));
         assert_eq!(iter.next(), None);
     }
+
+    #[test]
+    fn indexing_list() {
+        let list = Term::from(vec![zero(), one(), to_cnum(2), to_cnum(3), to_cnum(4)]);
+
+        assert_eq!(list[0], zero());
+        assert_eq!(list[1], one());
+        assert_eq!(list[2], to_cnum(2));
+        assert_eq!(list[3], to_cnum(3));
+        assert_eq!(list[4], to_cnum(4));
+    }
 }
